@@ -33,8 +33,8 @@ pub enum Oper {
   Xor,
   Shl,
   Shr,
-  Ltn,
   Lte,
+  Ltn,
   Eql,
   Gte,
   Gtn,
@@ -79,8 +79,8 @@ impl fmt::Display for Oper {
         Self::Xor => "^",
         Self::Shl => "<<",
         Self::Shr => ">>",
-        Self::Ltn => "<",
         Self::Lte => "<=",
+        Self::Ltn => "<",
         Self::Eql => "==",
         Self::Gte => ">=",
         Self::Gtn => ">",
@@ -286,19 +286,7 @@ pub fn parse_u32(state: parser::State) -> parser::Answer<Option<BTerm>> {
 
 pub fn parse_op2(state: parser::State) -> parser::Answer<Option<BTerm>> {
   fn is_op_char(chr: char) -> bool {
-    false
-      || chr == '+'
-      || chr == '-'
-      || chr == '*'
-      || chr == '/'
-      || chr == '%'
-      || chr == '&'
-      || chr == '|'
-      || chr == '^'
-      || chr == '<'
-      || chr == '>'
-      || chr == '='
-      || chr == '!'
+    matches!(chr, '+' | '-' | '*' | '/' | '%' | '&' | '|' | '^' | '<' | '>' | '=' | '!')
   }
   fn parse_oper(state: parser::State) -> parser::Answer<Oper> {
     fn op<'a>(symbol: &'static str, oper: Oper) -> parser::Parser<'a, Option<Oper>> {
@@ -320,8 +308,8 @@ pub fn parse_op2(state: parser::State) -> parser::Answer<Option<BTerm>> {
         op("^", Oper::Xor),
         op("<<", Oper::Shl),
         op(">>", Oper::Shr),
-        op("<", Oper::Ltn),
         op("<=", Oper::Lte),
+        op("<", Oper::Ltn),
         op("==", Oper::Eql),
         op(">=", Oper::Gte),
         op(">", Oper::Gtn),
@@ -404,15 +392,15 @@ pub fn parse_lst_sugar(state: parser::State) -> parser::Answer<Option<BTerm>> {
       let (state, _head) = parser::text("[", state)?;
       // let mut elems: Vec<Box<Term>> = Vec::new();
       let state = state;
-      let (state, elems) = 
-        parser::until(
-    Box::new(|x| parser::text("]", x)), 
-   Box::new(|x| { 
-            let (state, term) = parse_term(x)?;
-            let (state, _) = parser::maybe(Box::new(|x| parser::text(",", x)), state)?;
-            Ok((state, term))
-          })
-          , state)?;
+      let (state, elems) = parser::until(
+        Box::new(|x| parser::text("]", x)),
+        Box::new(|x| {
+          let (state, term) = parse_term(x)?;
+          let (state, _) = parser::maybe(Box::new(|x| parser::text(",", x)), state)?;
+          Ok((state, term))
+        }),
+        state,
+      )?;
       let empty = Term::Ctr { name: "Nil".to_string(), args: Vec::new() };
       let list = Box::new(elems.iter().rfold(empty, |t, h| Term::Ctr {
         name: "Cons".to_string(),
