@@ -32,7 +32,7 @@ mod tests {
     (Main) = (Fn 20)
     ";
 
-    let (norm, _cost, _size, _time) = eval_code(&make_call("Main", &[]), code);
+    let (norm, _cost, _size, _time) = eval_code(&make_call("Main", &[]), code, false);
     let norm = norm.to_string();
     assert_eq!(norm, "6765");
   }
