@@ -169,10 +169,8 @@ pub fn get_loc(lnk: Lnk, arg: u64) -> u64 {
 // ------
 
 pub fn ask_lnk(mem: &Worker, loc: u64) -> Lnk {
-  unsafe {
-    return *mem.node.get_unchecked(loc as usize);
-  }
-  //return mem.node[loc as usize];
+  unsafe { *mem.node.get_unchecked(loc as usize) }
+  // mem.node[loc as usize]
 }
 
 pub fn ask_arg(mem: &Worker, term: Lnk, arg: u64) -> Lnk {
@@ -196,10 +194,9 @@ pub fn link(mem: &mut Worker, loc: u64, lnk: Lnk) -> Lnk {
 pub fn alloc(mem: &mut Worker, size: u64) -> u64 {
   if size == 0 {
     0
+  } else if let Some(reuse) = mem.free[size as usize].pop() {
+    reuse
   } else {
-    if let Some(reuse) = mem.free[size as usize].pop() {
-      return reuse;
-    }
     let loc = mem.size;
     mem.size += size;
     loc
@@ -394,13 +391,13 @@ pub fn reduce(
         }
         DP0 | DP1 => {
           let arg0 = ask_arg(mem, term, 2);
-          //let argK = ask_arg(mem, term, if get_tag(term) == DP0 { 1 } else { 0 });
-          //if get_tag(argK) == ERA {
-            //let done = arg0;
-            //link(mem, host, done);
-            //init = 1;
-            //continue;
-          //}
+          // let argK = ask_arg(mem, term, if get_tag(term) == DP0 { 1 } else { 0 });
+          // if get_tag(argK) == ERA {
+          //   let done = arg0;
+          //   link(mem, host, done);
+          //   init = 1;
+          //   continue;
+          // }
           if get_tag(arg0) == LAM {
             //println!("dup-lam");
             inc_cost(mem);
@@ -512,48 +509,12 @@ pub fn reduce(
               XOR => (a ^ b) & 0xFFFFFFFF,
               SHL => (a << b) & 0xFFFFFFFF,
               SHR => (a >> b) & 0xFFFFFFFF,
-              LTN => {
-                if a < b {
-                  1
-                } else {
-                  0
-                }
-              }
-              LTE => {
-                if a <= b {
-                  1
-                } else {
-                  0
-                }
-              }
-              EQL => {
-                if a == b {
-                  1
-                } else {
-                  0
-                }
-              }
-              GTE => {
-                if a >= b {
-                  1
-                } else {
-                  0
-                }
-              }
-              GTN => {
-                if a > b {
-                  1
-                } else {
-                  0
-                }
-              }
-              NEQ => {
-                if a != b {
-                  1
-                } else {
-                  0
-                }
-              }
+              LTN => u64::from(a < b),
+              LTE => u64::from(a <= b),
+              EQL => u64::from(a == b),
+              GTE => u64::from(a >= b),
+              GTN => u64::from(a > b),
+              NEQ => u64::from(a != b),
               _ => 0,
             };
             let done = U_32(c);
@@ -736,7 +697,12 @@ pub fn show_mem(worker: &Worker) -> String {
   s
 }
 
-pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, String>>, focus: u64) -> String {
+pub fn show_term(
+  mem: &Worker,
+  term: Lnk,
+  opt_id_to_name: Option<&HashMap<u64, String>>,
+  focus: u64,
+) -> String {
   let mut lets: HashMap<u64, u64> = HashMap::new();
   let mut kinds: HashMap<u64, u64> = HashMap::new();
   let mut names: HashMap<u64, String> = HashMap::new();
@@ -873,10 +839,10 @@ pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, S
       _ => String::from("?"),
     };
     if term == focus {
-      return format!("${}", done);
+      format!("${}", done)
     } else {
-      return done;
-    };
+      done
+    }
   }
   find_lets(mem, term, &mut lets, &mut kinds, &mut names, &mut count);
   let mut text = go(mem, term, &names, opt_id_to_name, focus);
@@ -885,8 +851,10 @@ pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, S
     let what = String::from("?");
     //let kind = kinds.get(&key).unwrap_or(&0);
     let name = names.get(&pos).unwrap_or(&what);
-    let nam0 = if ask_lnk(mem, pos + 0) == Era() { String::from("*") } else { format!("a{}", name) };
-    let nam1 = if ask_lnk(mem, pos + 1) == Era() { String::from("*") } else { format!("b{}", name) };
+    let nam0 =
+      if ask_lnk(mem, pos + 0) == Era() { String::from("*") } else { format!("a{}", name) };
+    let nam1 =
+      if ask_lnk(mem, pos + 1) == Era() { String::from("*") } else { format!("b{}", name) };
     text.push_str(&format!(
       "\ndup {} {} = {};",
       //kind,
